<?php

namespace Ems\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\{ Error, Message };
use Ems\Util\ExpectedException;

use Ems\Services\AdmissaoService;

class AdmissaoController {
    /** @var AdmissaoService */
    protected $admissaoService;

    public function __construct(
        AdmissaoService $admissaoService
    ) {
        $this->admissaoService = $admissaoService;
    }

    public function getVinculosNaoAprovados(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $cdOperador = (string) $row['CDOPERADOR'];

            $nrEstrutura = isset($row['NRESTRUTURAM']) ?
                (int) $row['NRESTRUTURAM'] :
                null;

            $admissoes = $this->admissaoService->getVinculosNaoAprovados(
                $cdOperador, $nrEstrutura
            );

            $response->addDataSet(new DataSet('getVinculosNaoAprovados', $admissoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function setAdmissaoSimplificada(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = (array) $request->getRow()->getArrayCopy();

            $success = $this->admissaoService->setAdmissaoSimplificada(
                $row
            );

            $dataset = [
                'success' => $success,
            ];

            $response->addDataSet(new DataSet('setAdmissaoSimplificada', $dataset));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}
