<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class SolicitacaoRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->connection = $entityManager->getConnection();
    }

    public function getSolicitacoesVinculosGestor(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                SL.NRORG,
                SL.NRSOLICITACAO,
                SL.NRTIPOSOLICITACAO,
                SL.NRVINCULOM,
                TO_CHAR(SL.DTSOLICITACAO,'DD/MM/YYYY') AS DTSOLICITACAO,
                SL.IDAPROVACAOPRI,
                DECODE (SL.IDAPROVACAOPRI,  'A', 'Aprovada',
                                            'R', 'Rejeitada',
                                            'P', 'Pendente',
                                            'Pendente') DSAPROVACAOPRI,
                DECODE (SL.IDAPROVACAOSEC,  'A', 'Aprovada',
                                            'R', 'Rejeitada',
                                            'P', 'Pendente',
                                            'Pendente') DSAPROVACAOSEC,
                SL.DTAPROVACAOPRI,
                SL.CDOPERAPROVPRI,
                SL.IDAPROVACAOSEC,
                SL.DTAPROVACAOSEC,
                SL.CDOPERAPROVSEC,
                SL.DSOBSSOLICITACAO,
                SL.DSOBSAPROVAPRI,
                SL.DSOBSAPROVASEC,
                VM_S.NMVINCULOM,
                TP.NMTIPOSOLICITACAO,
                PS.NRNIVELAPROVASOLI,
                PS.NRPERFACESAPROVPRI,
                PS.NRPERFACESAPROVSEC
            FROM
                GPE_SOLICITACAO SL
                LEFT JOIN
                GPE_TIPOSOLICITACAO TP
                    ON SL.NRTIPOSOLICITACAO = TP.NRTIPOSOLICITACAO
                LEFT JOIN
                GPE_PERFACESTIPOSOLI PS
                    ON PS.NRORG = SL.NRORG
                   AND PS.NRTIPOSOLICITACAO = SL.NRTIPOSOLICITACAO
                LEFT JOIN
                GPE_VINCULOM VM_S
                    ON VM_S.NRORG = SL.NRORG
                   AND VM_S.NRVINCULOM = SL.NRVINCULOM
                LEFT JOIN
                GPE_VINCULOH VH_S
                    ON VH_S.NRORG = VM_S.NRORG
                   AND VH_S.NRVINCULOM = VM_S.NRVINCULOM
                   AND VH_S.DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            GPE_VINCULOH V1
                        WHERE
                            V1.NRORG = VH_S.NRORG
                        AND V1.NRVINCULOM = VH_S.NRVINCULOM
                        AND V1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                    )
            WHERE
                (
                    SL.NRORG = :P_NRORG
                    OR SL.NRORG = 0
                )
            AND (INSTR(:P_NRESTRUTURAS, ',' || VH_S.NRESTRUTGEREN || ',')) > 0
            ORDER BY NMVINCULOM ASC, SL.DTSOLICITACAO ASC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $solicitacoes = $std->fetchAll();

        return $solicitacoes;
    }

    public function getItemSolicitacao($nrorg, $nrsolicitacao ) {
        $sql = <<<SQL
    SELECT
        IT.NRORG,
        IT.NRITEMSOLICITACAO,
        IT.NRSOLICITACAO,
        IT.DSCHAVEITEMSOLI,
        IT.DSLABELITEMSOLI,
        IT.DSVALORITEMSOLI,
        IT.DTINCLUSAO,
        IT.DTULTATU,
        IT.NRORGINCLUSAO,
        IT.CDOPERINCLUSAO,
        IT.NRORGULTATU,
        IT.CDOPERULTATU,
        TP.NMTIPOSOLICITACAO
    FROM GPE_ITEMSOLICITACAO IT
    JOIN
          GPE_SOLICITACAO SL
          ON SL.NRSOLICITACAO = IT.NRSOLICITACAO
          AND SL.NRORG = IT.NRORG

    LEFT JOIN
          GPE_TIPOSOLICITACAO TP
          ON SL.NRTIPOSOLICITACAO = TP.NRTIPOSOLICITACAO

    WHERE
        IT.NRORG = :P_NRORG
        AND IT.NRSOLICITACAO = :P_NRSOLICITACAO
SQL;


        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRSOLICITACAO'   => $nrsolicitacao
            )
        );

        $resultQuery = $query->fetchAll();


        if (isset($resultQuery)) {
            return $resultQuery;
        } else {
            return null;
        }
    }


    public function getTiposolicitacao($nrorg) {
        $sql = <<<SQL
    SELECT
   *
    FROM
       GPE_TIPOSOLICITACAO
    WHERE
       (
          NRORG = :P_NRORG
          OR NRORG = 0
       )
        AND NRTIPOSOLICITACAO IN (3, 5, 9, 11, 14)
SQL;


        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg
            )
        );

        $resultQuery = $query->fetchAll();


        if (isset($resultQuery)) {
            return $resultQuery;
        } else {
            return null;
        }
    }
    
    public function getSolicitacao($nrorg, $nrsolicitacao = null) {
        $sql = <<<SQL
    SELECT
        *
    FROM
        GPE_SOLICITACAO SL
    JOIN GPE_TIPOSOLICITACAO TS ON (TS.NRORG = SL.NRORG OR TS.NRORG = 0) AND TS.NRTIPOSOLICITACAO = SL.NRTIPOSOLICITACAO
    WHERE
        SL.NRORG = :P_NRORG
        AND (SL.NRSOLICITACAO = :P_NRSOLICITACAO OR :P_NRSOLICITACAO IS NULL)
SQL;


        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRSOLICITACAO'   => $nrsolicitacao,
            )
        );

        $resultQuery = empty($nrsolicitacao) ? $query->fetchAll() : $query->fetch();
        
        return $resultQuery;
    }

    public function getTipoRelacionamento(
        int $nrOrg,
        int $nrOrgPadrao
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                TIPORELACIONA
            WHERE
                NRORG = :P_NRORG OR
                NRORG = :P_NRORG_PADRAO
            ORDER BY
                NMTIPORELACIONA ASC
SQL;


        $std = $this->connection->prepare($sql);

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
        ];

        $std->execute($params);
        $tiposRel = $std->fetchAll();

        return $tiposRel;
    }

}