<?php

namespace Ems\Services;

use Util\NovoCodigo;
use Util\Environment;
use Ems\Util\{ ExpectedException, Utilities };
use Ems\Repository\DocumentoRepository;
use Ems\Services\VinculoService;

class DocumentoService {
    /** @var Environment */
    protected $environment;

    /** @var DocumentoRepository */
    protected $documentoRepository;

    /** @var Utilities */
    protected $utilities;

    /** @var VinculoService */
    protected $vinculoService;

    public function __construct(
        Environment         $environment,
        DocumentoRepository $documentoRepository,
        Utilities           $utilities,
        VinculoService      $vinculoService
    ) {
        $this->environment          = $environment;
        $this->documentoRepository  = $documentoRepository;
        $this->utilities            = $utilities;
        $this->vinculoService       = $vinculoService;
    }

    public function getDocumentosVinculo(
        int $nrVinculo,
            $nrCategoria = null
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $params = [
            'P_NRORG'       => $nrOrg,
            'P_NRVINCULO'   => $nrVinculo,
            'P_NRCATEGORIA' => $nrCategoria,
        ];

        $documentos = $this->documentoRepository->getDocumentosVinculo(
            $params
        );

        return $documentos;
    }

    public function getCategoriasComDocumentoVinculo(
        int $nrVinculo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $params = [
            'P_NRORG'     => $nrOrg,
            'P_NRVINCULO' => $nrVinculo,
        ];

        $categorias = $this->documentoRepository->getCategoriasComDocumentoVinculo(
            $params
        );

        return $categorias;
    }

    public function getCategorias(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $params = [
            'P_NRORG'     => $nrOrg,
        ];

        $categorias = $this->documentoRepository->getCategorias(
            $params
        );

        return $categorias;
    }

    public function getTipos(): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $params = [
            'P_NRORG'     => $nrOrg,
        ];

        $tipos = $this->documentoRepository->getTipos(
            $params
        );

        return $tipos;
    }

    public function downloadDocumento(
        int $nrAnexo
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $documento = $this->documentoRepository->get(
            $nrOrg, $nrAnexo
        );

        if (!$documento) {
            $message = 'O documento informado não foi encontrado!';
            throw new ExpectedException($message);
        }

        $path = (string) $documento['DSPATH'];
        $file = $this->utilities->downloadFile($path);

        return $file;
    }

    public function saveDocumento(
        array $row
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrParcNegocio = null;

        if (isset($row['NRPARCNEGOCIO'])) {
            $nrParcNegocio = $row['NRPARCNEGOCIO'];
        } else {
            $nrVinculo = (int) $row['NRVINCULOM'];

            $nrParcNegocio = $this->vinculoService->getNrParcNegocioVinculo(
                $nrVinculo
            );
        }

        // Salva o arquivo no File Server
        $filePath = $this->saveFileOnServer($row);

        $nrAnexo = NovoCodigo::geraCodigo(
            $nrOrg, 'GPE_ANEXOS', 1, 10, true
        );

        $base64 = (string) $row['FILE'];
        $type = $this->utilities->getFileMIMEType($base64);

        $params = [
            'DSPATH'        => $filePath,
            'DSREFERENCIA'  => $row['DSREFERENCIA'],
            'IDFOTO'        => 'N',
            'IDTYPE'        => $type,
            'NMANEXO'       => $row['NMANEXO'],
            'NRANEXO'       => $nrAnexo,
            'NRCATEGORIA'   => $row['NRCATEGDOCUMENTO'],
            'NRORG'         => $nrOrg,
            'NRPARC'        => $nrParcNegocio,
            'NRTIPO'        => $row['NRTIPODOCUMENTO'],
        ];

        try {
            $this->documentoRepository->saveDocumento($params);
        } catch (\Exception $e) {
            $this->utilities->deleteFile($filePath);
            throw $e;
        }
    }

    private function saveFileOnServer(
        array $row
    ): string {
        $nrOrg = (int) $this->environment->getNrOrg();

        $base64 = (string) $row['FILE'];

        $directory = "EMS/$nrOrg/Documentos/";
        $name = (string) $row['NMANEXO'];

        $filePath = $this->utilities->saveFileOnFileServer(
            $base64, null, $nrOrg, $directory,
            $name, null, false
        );

        return $filePath;
    }
}