<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Environment;
use Util\DateUtil;
use Util\Repositories;

use Ems\Services\EstruturaService;
use Ems\Repository\SolicitacaoRepository;
use Ems\Util\{ Utilities, ArrayUtil };
use Ems\Repository\{ PeriodoRepository, SituacaoFuncionalRepository };

class SituacaoFuncionalService {

    public function __construct(
        EntityManager                   $entityManager,
        Environment                     $environment,
        SituacaoFuncionalRepository     $situacaoFuncionalRepository,
        PeriodoRepository               $periodoRepository
    ) {
        $this->entityManager                = $entityManager;
        $this->environment                  = $environment;
        $this->situacaoFuncionalRepository  = $situacaoFuncionalRepository;
        $this->periodoRepository            = $periodoRepository;
    }

  /**
    * Rotina responsável por obter a última situação funcional de um determinado vínculo em um período de apuração ou dentro da competência. Caso
    * seja informado um período somente, a data de competência é obtida a partir dele
    * 
    * @param    int     $nrVinculom 
    * @param    int     $nrPeriodoApuracao  = null
    * @param    string  $mesCompetencia     = null
    * 
    * @return   Array   SituacaoFuncionalRepository->retornaUltimaSituacaoFuncional
    */
    public function retornaUltimaSituacaoFuncional(
        int     $nrVinculom,
        string  $nrPeriodoApuracao = null,
        string  $mesCompetencia = null
    ): Array {

        $nrOrg          = $this->environment->getNrorg();
        $nrOrgPadrao    = $this->environment->getNrorgPadrao();

        if (empty($nrPeriodoApuracao) && empty($mesCompetencia)) {
            return false;
        }

        $dtCompetenciaSituacaoFuncional = $mesCompetencia;
        if (empty($mesCompetencia)) {
            $dtCompetenciaSituacaoFuncional = $this->periodoRepository->getDtmescompetencByPeriodo($nrOrg, $nrPeriodoApuracao);
        }

        return $this->situacaoFuncionalRepository->retornaUltimaSituacaoFuncional(
            $nrOrg,
            $nrOrgPadrao,
            $nrVinculom,
            $dtCompetenciaSituacaoFuncional    
        );
    }
    
  /**
    * Rotina responsável por obter os períodos de estabilidade de um determinado vínculo
    * 
    * @param    int     $nrVinculom 
    * @param    int     $nrPeriodoApuracao  = null
    * @param    string  $mesCompetencia     = null
    * 
    * @return   Array   SituacaoFuncionalRepository->retornaPeriodosEstabilidade
    */
    public function retornaPeriodosEstabilidade(
        int     $nrVinculom
    ): Array {

        $nrOrg          = $this->environment->getNrorg();
        $nrOrgPadrao    = $this->environment->getNrorgPadrao();

        return $this->situacaoFuncionalRepository->retornaPeriodosEstabilidade(
            $nrOrg,
            $nrOrgPadrao,
            $nrVinculom
        );
    }

}
