<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Environment;
use Util\DateUtil;

use Ems\Services\EstruturaService;
use Ems\Repository\SolicitacaoRepository;
use Ems\Util\{ Utilities, ArrayUtil };

class SolicitacaoService {

    /** @var EstruturaService */
    protected $estruturaService;

    /** @var Environment */
    protected $environment;

    /** @var SolicitacaoRepository */
    protected $solicitacaoRepository;

    public function __construct(
        SolicitacaoRepository   $solicitacaoRepository,
        Environment             $environment,
        EstruturaService        $estruturaService
    ) {
        $this->solicitacaoRepository    = $solicitacaoRepository;
        $this->environment              = $environment;
        $this->estruturaService         = $estruturaService;
    }

    public function getSolicitacoesVinculosGestor(
        int $nrEstrutura
    ): array {
        $nrOrg = (int) $this->environment->getNrorg();
        $dtmescompetenc = DateUtil::getPrimeiroDiaDoMes(new \Setforms2\Shared\ORM\DateTime())->format(DateUtil::FORMATO_BRASILEIRO);

        $nrEstruturas = $this->estruturaService->
            getEstruturasInferiores($nrEstrutura, true);

        $nrEstruturasString = ArrayUtil::toStringSQL($nrEstruturas);

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRESTRUTURAS'    => $nrEstruturasString,
            'P_DTMESCOMPETENC'  => $dtmescompetenc
        ];

        $solicitacoes = $this->solicitacaoRepository->
            getSolicitacoesVinculosGestor($params);

        return $solicitacoes;
    }

    public function getItemSolicitacao($nrsolicitacao) {
        $nrorg = $this->environment->getNrorg();
        return $this->solicitacaoRepository->getItemSolicitacao($nrorg, $nrsolicitacao);
    }
    
    public function getSolicitacao($nrsolicitacao) {
        $nrorg = $this->environment->getNrorg();
        return $this->solicitacaoRepository->getSolicitacao($nrorg, $nrsolicitacao);
    }

    public function getTiposolicitacao() {
        $nrorg = $this->environment->getNrorg();
        return $this->solicitacaoRepository->getTiposolicitacao($nrorg);
    }

    public function getTipoRelacionamento() {
        $nrOrg = (int) $this->environment->getNrorg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $tiposRel = $this->solicitacaoRepository->getTipoRelacionamento(
            $nrOrg, $nrOrgPadrao
        );

        return $tiposRel;
    }
}
